#ifndef BOOST_STATECHART_HISTORY_HPP_INCLUDED
#define BOOST_STATECHART_HISTORY_HPP_INCLUDED
//////////////////////////////////////////////////////////////////////////////
// Copyright 2002-2006 Andreas Huber Doenni
// Distributed under the Boost Software License, Version 1.0. (See accompany-
// ing file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
//////////////////////////////////////////////////////////////////////////////



#include <boost/statechart/shallow_history.hpp>
#include <boost/statechart/deep_history.hpp>



#endif
